#!/usr/bin/env python
################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
################################################################################
# glite-file-transfer-service-client-config v. 1.2.1
#
# Post-installation script for configuring the gLite File Transfer Service client
#
# Joachim Flammer <Joachim.Flammer@cern.ch>
#
# Version info: $Id: glite-file-transfer-service-client-config.py,v 1.3.2.2 2006/03/02 00:02:47 dimeglio Exp $
#
# Usage: python glite-file-transfer-service-client-config [-c|-v|-h|--help]
#        -c,--checkconf       print configuration
#        -v,--version       print version
#        -h,--help          print usage info
#               --configure        configure worker node
#               --siteconfig=URL   specify URL of the site configuration file
#        --subservice       run as subservice script
#
# Return codes: 0 - Ok
#            1 - Configuration failed
#
################################################################################

import os,string,socket,pwd
import sys
import getopt
import commands
from xml.dom.minidom import getDOMImplementation
from gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams
from serviceDiscovery import ServiceDiscovery

# Set global variables here 
global params                         # all config values from the XML file

class glite_file_transfer_service_client:

    #################################################################
    # init 
    #
    # init
    #
    #################################################################        
    def __init__(self):
        self.verbose = 0
        self.version = "1.2.1"
        self.name = "glite-file-transfer-service-client"
        self.friendly_name = "gLite File Transfer Service Client"
        

    #################################################################
    # banner 
    #
    # print banner
    #
    #################################################################    
    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "        Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #################################################################
    # copyright 
    #
    # print copyright
    #
    #################################################################    
    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'


    #################################################################
    # version
    #
    # print version
    #
    #################################################################    
    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    

    #################################################################
    # usage 
    #
    # print usage
    #
    #################################################################    
    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()

        print "Usage: \n"
        print "Edit the configuration file %s.cfg.xml in " %  self.name,
        print "%s/etc.config/templates" % os.environ['GLITE_LOCATION']
        print "save it as %s/etc/config/%s.cfg.xml" % (os.environ['GLITE_LOCATION'], self.name),
        print "and run the script as follows"
        print "python %s-config [OPTION...]" % self.name
        print ''
        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --configure         configure %s' % self.friendly_name
        print '    --subservice        run script as a subservice'
        print ''
        print '    --siteconfig=URL    URL of the site configuration file'
        print ''
        print '\n'
    

    #################################################################
    # configure
    #
    # configure file transfer service client
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configure(self):

        # ------------------------------------------------------------
        # get corresponding configuration values
        # ------------------------------------------------------------
        
        errorFlag = 0
        
        # subservice                    
        if params.has_key('fts.client.run_as_subservice'):
            subservice = params['fts.client.run_as_subservice']
        else:
            subservice = "false"            

        # ------------------------------------------------------------        
        # Check certificates
        # ------------------------------------------------------------    
        if subservice=="false":
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        print "An error occurred while configuring the %s service" \
                            % service.friendly_name
                        sys.exit(1) 
    
        # ------------------------------------------------------------
        # Run security util script
        # ------------------------------------------------------------
                
        if subservice=="false":
            print "Configuring security ...                                 "
            errorcode = os.system("%s/etc/config/scripts/glite-security-utils-config.py --subservice" % os.environ['GLITE_LOCATION'])
    
            if errorcode:
                print "Configuring security                                      ",
                glib.printFailedMessage()
                return 1
        
            print "Configuring security                                      ",
            glib.printOkMessage()
                                    
        # ------------------------------------------------------------
        # configure service discovery
        # ------------------------------------------------------------

        errorcode =    serviceDiscovery.configure(params,glib)
        if errorcode:
            return 1

        return 0
        

    #################################################################
    # createParameterNode
    #
    # create a node in xml dom tree document
    #
    # params  factory   dom tree document
    #         name      name of node
    #         value     value of node
    # return parameter node
    #
    #################################################################
    def createParameterNode(self, factory, name, value):

        node = factory.createElement(name)
        nodeText = factory.createTextNode(value)
        node.appendChild(nodeText)
        
        return node


#################################################################
# set_env
#
# setup environment
# 
#################################################################
def set_env():

    if not os.geteuid() == 0:
        
        if not os.access(params['installer.export.filename'], os.W_OK):
            glib.check_dir("%s/.glite" % os.environ['HOME'])
            params['installer.export.filename'] = "%s/.glite/glite_setenv.sh" % os.environ['HOME']
        
        glib.printInfoMessage("gLite environment file will be saved to %s" % params['installer.export.filename'])
        
        installPrefix = os.path.abspath(glib.getScriptPath() + "/../../../")
        
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/lib' % installPrefix)
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/local/lib' % installPrefix)
        
        glib.addEnvPath('PATH','%s/usr/bin' % installPrefix)
        
        glib.addEnvPath('PERL5LIB', "%s/usr/lib/perl5/site_perl/5.6.0/" % installPrefix)
        glib.addEnvPath('PERL5LIB', "%s/usr/lib/perl5/vendor_perl/5.8.0/" % installPrefix)
        
        glib.printWarningMessage("The configuration script can modify the environment variables")
        print "    Modifications are listed below\n"
        
        if os.path.exists(params['GLITE_LOCATION']):
            if not os.access(params['GLITE_LOCATION'], os.W_OK):
                print "    GLITE_LOCATION is set to the non-writable directory %s -" % params['GLITE_LOCATION'],
                params['GLITE_LOCATION'] = "%s" % (installPrefix)
                print "it will be changed to %s" % params['GLITE_LOCATION']
            else:
                if not params['GLITE_LOCATION'] == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION'] = "%s" % (installPrefix)

        if not os.path.exists(params['GLOBUS_LOCATION']):
            print "    GLOBUS_LOCATION is set to the non-existing directory %s " % (params['GLOBUS_LOCATION']),
            params['GLOBUS_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['GLOBUS_LOCATION']

        if not os.path.exists(params['GPT_LOCATION']):
            print "    GPT_LOCATION is set to the non-existing directory %s " % (params['GPT_LOCATION']),
            params['GPT_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['GPT_LOCATION']

        if os.path.exists(params['GLITE_LOCATION_VAR']):
            if not os.access('GLITE_LOCATION_VAR', os.W_OK):
                print "    GLITE_LOCATION_VAR is set to the non-writable directory %s - " % params['GLITE_LOCATION_VAR'],
                params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
                print "value changed to %s" % params['GLITE_LOCATION_VAR']
            else:
                if not params['GLITE_LOCATION_VAR'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
        
        if os.path.exists(params['GLITE_LOCATION_LOG']):
            if not os.access('GLITE_LOCATION_LOG', os.W_OK):
                print "    GLITE_LOCATION_LOG is set to the non-writable directory %s - " % params['GLITE_LOCATION_LOG'],
                params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
                print "it will be changed to %s" % params['GLITE_LOCATION_LOG']
            else:
                if not params['GLITE_LOCATION_LOG'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
        
        if os.path.exists(params['ca.certificates.dir']):
            if not os.access('ca.certificates.dir', os.W_OK):
                print "    ca.certificates.dir is set to the non-writable directory %s - " % params['ca.certificates.dir'],
                params['ca.certificates.dir'] = "%s/etc/grid-security/certificates" % installPrefix
                print "it will be changed to %s" % params['ca.certificates.dir']
            else:
                if not params['ca.certificates.dir'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
        
        print "\n"

    # Standard configuration
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    glib.export('GLITE_LOCATION_LOG');
    glib.export('GLITE_LOCATION_TMP');
    glib.check_dir(os.environ['GLITE_LOCATION_VAR'])
    glib.check_dir(os.environ['GLITE_LOCATION_LOG'])
    glib.check_dir(os.environ['GLITE_LOCATION_TMP'])
    
    glib.export('GLOBUS_LOCATION');
    glib.export('GPT_LOCATION');
    glib.export('JAVA_HOME');

    # bin and lib paths
    glib.addEnvPath("PATH","/usr/bin")
    glib.addEnvPath("PATH","$JAVA_HOME/bin")
    glib.addEnvPath("PATH","$GLOBUS_LOCATION/bin")
    glib.addEnvPath("PATH","$GLITE_LOCATION/externals/bin")
    glib.addEnvPath("PATH","$GLITE_LOCATION/bin")

    glib.addEnvPath("LD_LIBRARY_PATH","/usr/local/lib")
    glib.addEnvPath("LD_LIBRARY_PATH","$GLOBUS_LOCATION/lib")
    glib.addEnvPath("LD_LIBRARY_PATH","$GLITE_LOCATION/externals/lib")
    glib.addEnvPath("LD_LIBRARY_PATH","$GLITE_LOCATION/lib")

    # Perl
    glib.addEnvPath("PERL5LIB", "$GPT_LOCATION/lib/perl")
    glib.addEnvPath("PERL5LIB", "$GLITE_LOCATION/lib/perl5")

    # MANPATH
    glib.addEnvPath('MANPATH', '$GLOBUS_LOCATION/man')
    glib.addEnvPath('MANPATH', '$GLITE_LOCATION/share/man')

    # Set environment
    glib.setUserEnv()

#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # Get an instance of the library class
    glib = gLib()
    
    # ------------------------------------------------------------
    # Load parameters
    # ------------------------------------------------------------
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass
        
    # load fts client configuration file
    if glib.loadConfiguration("%s/../glite-file-transfer-service-client.cfg.xml" % glib.getScriptPath(),params):
        glib.printErrorMessage("Could not load configuration file %s/../glite-file-transfer-service-client.cfg.xml" % glib.getScriptPath())
        sys.exit(1)
    
    # load service-discovery configuration file    
    if glib.loadConfiguration("%s/../glite-service-discovery.cfg.xml" % glib.getScriptPath(),params):
        glib.printErrorMessage("Could not load configuration file %s/../glite-service-discovery.cfg.xml" % glib.getScriptPath())
        sys.exit(1)
    
    # ------------------------------------------------------------    
    # set verbosity
    # ------------------------------------------------------------    
    
    verbose = 0
    if params.has_key('glite.installer.verbose'):
        if params['glite.installer.verbose'] == "true":
            verbose = 1
    glib.verbose = verbose
    
    # ------------------------------------------------------------    
    # Set up the environment
    # ------------------------------------------------------------    
    set_env()
          
    # ------------------------------------------------------------    
    # Instantiate the service classes
    # ------------------------------------------------------------    
    service = glite_file_transfer_service_client()
    service.verbose = verbose

    serviceDiscovery = ServiceDiscovery()
    serviceDiscovery.verbose = verbose
    
    # ------------------------------------------------------------
    # Command line opts if any
    # ------------------------------------------------------------
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version','siteconfig=', 'configure', 'subservice'])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    # ------------------------------------------------------------
    # Check cli options
    # ------------------------------------------------------------
    configure  = 0
    subservice = 0
    
    for o, a in opts:
        # help
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
            
        # show version
        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)
        
        # check configuration
        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)

        # configure service
        if o == "--configure":
            configure = 1

        # configure service
        if o == "--subservice":
            subservice = 1
            params['fts.client.run_as_subservice'] = "true"

    #-------------------------------------------------------------------------------    
    # Configure
    #-------------------------------------------------------------------------------    
    if configure==1:
        
        #-------------------------------------------------------------------------------    
        # Print configuration parameters
        #-------------------------------------------------------------------------------    
        if verbose:
            glib.print_params(params)

        #-------------------------------------------------------------------------------
        # print default statements
        #-------------------------------------------------------------------------------    
        if subservice==0:
            service.copyright()
            service.showVersion()
       
        # print banner
        service.banner()
    
        #-------------------------------------------------------------------------------
        # Configure the FTS client
        #-------------------------------------------------------------------------------    
        if service.configure() != 0:
            glib.printErrorMessage("An unrecoverable error occurred while configuring the %s service" % service.friendly_name)
            sys.exit(1)
        else:
            print "\n%s configuration successfully completed\n" % service.friendly_name            
            sys.exit(0)
            
    else:
        # ------------------------------------------------------------
        # No option given
        # ------------------------------------------------------------
        service.usage()
        sys.exit(0)            
    
